{ open Parser }

let un_letter = ['a' - 'z']
let cap_letter = ['A' - 'Z']
let letter = ['a' - 'z' 'A' - 'Z']
let digit = ['0' - '9']
let duration = 'w'|'h'|'q'|'i'|'s'|'t'

rule token = parse
  [' ' '\t' '\r'] { token lexbuf }
| "/*"        { comment lexbuf }
| '('         { LPAREN }
| ')'         { RPAREN }
| '{'         { LBRACE }
| '}'         { RBRACE }
| '\n'        { EOL }
| '+'         { PLUS }
| '-'         { MINUS }
| '*'         { TIMES }
| '/'         { DIVIDE }
| '%'         { MODULUS }
| '<'         { LESS }
| '>'         { GREATER }
| '='         { ASSIGN }
| ':'		      { COLON }
| ';' 		    { SEMI }
| '.'		      { DOT }
| "<="        { LEQ }
| ">="        { GEQ }
| "=="        { EQ }
| "!="        { NEQ }
| "&&"        { AND }
| "||"        { OR }
| "!"		      { NOT }
| "["         { LBRACKET }
| "]"         { RBRACKET }
| ","         { COMMA }
| "if"        { IF }
| "else"	    { ELSE }
| "while"     { WHILE }
| "for" 	    { FOR }
| "return"    { RETURN }
| "bool"      { BOOL }
| "float"     { FLOAT }
| "string"    { STRING }
| "int"       { INT }
| "void"	    { VOID }
| "note"	    { NOTE }
| "duration"  { DURATION }
| "tune"      { TUNE }
| "char"      { CHAR }
| "true"      { TRUE }
| "false"     { FALSE }
| eof         { EOF }
| (['a'-'g'] ('b'|'#')? (['1'-'9'])?) | 'r' as note { NOTELIT note }
| duration+ as dur { DURATIONLIT dur }
| digit* '.' digit* (('e'|'E')('+'|'-')?digit+)? as flt { FLOATLIT(float_of_string flt)}
| "\"" ([^ '\"' ]* as str) "\"" { STRINGLIT(Scanf.unescaped str) }
| digit+ as integer { INTLIT(int_of_string integer) }
| letter(letter | digit | '_')* as lxm { ID(lxm) }

and comment = parse
 "*/" { token lexbuf }
 | _ { comment lexbuf }

